#include "mode3.h"
#include "bios_arm.h"


//---------------------------------------------------------------------------
ST_MODE3 Mode3 EWRAM_BSS;


//---------------------------------------------------------------------------
EWRAM_CODE void Mode3Init(void)
{
	BiosExecFixClear(&Mode3, sizeof(ST_MODE3));

	REG_DISPCNT = (MODE_3 | BG2_ON);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Exec(void)
{
	Mode3ExecBuf();
	Mode3ExecFace();
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3ExecBuf(void)
{
	if(Mode3.isDrawBuf == FALSE)
	{
		return;
	}

	BiosExecFast(Mode3.buf, (u16*)VRAM, MODE3_MAX_SCREEN_SIZE);
	Mode3.isDrawBuf = FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3ExecFace(void)
{
	if(Mode3.isDrawFace == FALSE)
	{
		return;
	}

	u16* pS = Mode3.face;
	u16* pD = ((u16*)VRAM) + ((SCREEN_CY - 64) * SCREEN_CX);
	u32  y;

	for(y=0; y<64; y++)
	{
		BiosExec(pS, pD, 64*2);

		pS += 64;
		pD += SCREEN_CX;
	}

	Mode3.isDrawFace = FALSE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBg(u16* pImg)
{
	LZ77UnCompVram(pImg, Mode3.buf);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBgSmall(u16* pImg)
{
	LZ77UnCompVram(pImg, Mode3.buf+240*30);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawFill(u16 col)
{
	u32 c =((u32)col << 16) | (u32)col;
	BiosExecFastFix(&c, Mode3.buf, MODE3_MAX_BUF_SIZE);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawFace(u16* pImg)
{
	LZ77UnCompVram(pImg, Mode3.face);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawCrop(u32 sx, u32 sy, u32 cx, u32 cy, u16* pImg)
{
	LZ77UnCompVram(pImg, Mode3.buf + SCREEN_CX * SCREEN_CY);

	u16* pS = Mode3.buf + SCREEN_CX * SCREEN_CY;
	u16* pD = Mode3.buf + (sy * SCREEN_CX) + sx;
	u32  y;

	for(y=0; y<cy; y++)
	{
		BiosExec(pS, pD, cx*2);

		pS += cx;
		pD += SCREEN_CX;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawClip(u32 sx, u32 sy, u32 cx, u32 cy, u32 gx, u32 gy, u32 gs, u16* pImg)
{
	LZ77UnCompVram(pImg, Mode3.buf + SCREEN_CX * SCREEN_CY);

	u16* pS = Mode3.buf + SCREEN_CX * SCREEN_CY + (gy * cx) + gx;
	u16* pD = Mode3.buf + (sy * SCREEN_CX) + sx;
	u32  x, y;

	for(y=0; y<cy-gy; y++)
	{
		for(x=0; x<gs; x++)
		{
			*pD++ = *pS++;
		}

		pS += cx - gs;
		pD += SCREEN_CX - gs;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBlend(u32 sx, u32 sy, u32 cx, u32 cy, u16* pImg, u16* pMask)
{
	LZ77UnCompVram(pImg, Mode3.buf + SCREEN_CX * SCREEN_CY);
	LZ77UnCompVram(pMask, Mode3.mask);

	u16* pC = Mode3.buf + SCREEN_CX * SCREEN_CY;
	u8*  pM = Mode3.mask;
	u16* pV = Mode3.buf + sy * SCREEN_CX + sx;
	u32 x, y;

	for(y=sy; y<sy+cy; y++)
	{
		for(x=sx; x<sx+cx; x++)
		{
			if(*pM == 0x00)
			{
				// EMPTY
			}
			else if(*pM == 0x1f)
			{
				*pV = *pC;
			}
			else
			{
				u32 sR = (*pC & 0x001f);
				u32 sG = (*pC & 0x03e0);
				u32 sB = (*pC & 0x7c00);

				u32 dR = (*pV & 0x001f);
				u32 dG = (*pV & 0x03e0);
				u32 dB = (*pV & 0x7c00);

				u32 r = ((sR - dR) * *pM) / 32 + dR;
				u32 g = ((sG - dG) * *pM) / 32 + dG;
				u32 b = ((sB - dB) * *pM) / 32 + dB;

				*pV = (b & 0x7c00) | (g & 0x03e0) | (r & 0x001f);
			}

			pM++;
			pC++;
			pV++;
		}

		pV += SCREEN_CX - cx;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawLine(u32 cnt, u16* pImg)
{
	BiosExec(pImg + (162 * cnt), (u16*)VRAM + (SCREEN_CX * (SCREEN_CY-1)) + 40, 162*2);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Copy(u32 sx, u32 sy, u32 ex, u32 ey)
{
	_ASSERT(sx < SCREEN_CX);
	_ASSERT(sy < SCREEN_CY);

	u32 x, y;

	for(y=sy; y<=ey; y++)
	{
		for(x=sx; x<=ex; x++)
		{
			((u16*)VRAM)[(y * SCREEN_CX) + x] = Mode3.buf[(y * SCREEN_CX) + x];
		}
	}

	_ASSERT(ex < SCREEN_CX);
	_ASSERT(ey < SCREEN_CY);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Scroll(u32 cnt)
{
	BiosExecFast(Mode3.buf + SCREEN_CX * cnt, (u16*)VRAM, MODE3_MAX_SCREEN_SIZE);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3ScrollEd(void)
{
	BiosExecFast((u16*)VRAM + 240, (u16*)VRAM, MODE3_MAX_SCREEN_SIZE - 240*2);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3SetDrawBuf(void)
{
	Mode3.isDrawBuf = TRUE;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3SetDrawFace(void)
{
	Mode3.isDrawFace = TRUE;
}
